@@ -48,10 +48,29 @@ def _run_shell_command(cmd, output=None, raise_on_error=True):
         return out[0].strip().decode()
 
 
-def get_login_password(site_name="github.com", netrc_file="~/.netrc"):
+def get_login_password(site_name="github.com",
+                       netrc_file="~/.netrc",
+                       git_credential_file="~/.git-credentials"):
     """Read a .netrc file and return login/password for LWN."""
-    n = netrc.netrc(os.path.expanduser(netrc_file))
-    return n.hosts[site_name][0], n.hosts[site_name][2]
+    try:
+        n = netrc.netrc(os.path.expanduser(netrc_file))
+    except OSError:
+        pass
+    else:
+        if site_name in n.hosts:
+            return n.hosts[site_name][0], n.hosts[site_name][2]
+
+    try:
+        with open(os.path.expanduser(git_credential_file)) as f:
+            for line in f:
+                parsed = parse.urlparse(line.strip())
+                if parsed.hostname == site_name:
+                    return (parse.unquote(parsed.username),
+                            parse.unquote(parsed.password))
+    except OSError:
+        pass
+
+    return None, None
 
 
 def git_remote_matching_url(url):
@@ -105,7 +124,7 @@ def get_github_hostname_user_repo_from_url(url):
         if "@" in host:
             username, sep, host = host.partition("@")
     else:
-        path = parsed.path[1:]
+        path = parsed.path[1:].rstrip('/')
         host = parsed.netloc
     user, repo = path.split("/", 1)
     return host, user, repo[:-4] if repo.endswith('.git') else repo
@@ -168,7 +187,8 @@ def git_pull_request(target_remote=None, target_branch=None,
                      rebase=True,
                      force_editor=False,
                      download=None,
-                     ignore_tag=False):
+                     ignore_tag=False,
+                     dont_fork=True):
     branch = git_get_branch_name()
     if not branch:
         LOG.critical("Unable to find current branch")
@@ -210,13 +230,13 @@ def git_pull_request(target_remote=None, target_branch=None,
     LOG.debug("GitHub user and repository to fork: %s/%s on %s",
               user_to_fork, reponame_to_fork, hostname)
 
-    try:
-        user, password = get_login_password(hostname)
-    except KeyError:
+    user, password = get_login_password(hostname)
+    if not user or not password:
         LOG.critical(
             "Unable to find your GitHub credentials for %s.\n"
             "Make sure you have a line like this in your ~/.netrc file:\n"
-            "machine %s login <login> password <pwd>",
+            "machine %s login <login> password <pwd>, "
+            "or use git store credentials",
             hostname, hostname
         )
         return 35
@@ -237,7 +257,7 @@ def git_pull_request(target_remote=None, target_branch=None,
         fork_and_push_pull_request(g, repo, rebase, target_remote,
                                    target_branch, branch, user, title, message,
                                    comment_on_update, comment,
-                                   force_editor, ignore_tag)
+                                   force_editor, ignore_tag, dont_fork)
 
 
 def download_pull_request(g, repo, target_remote, pull_number):
@@ -310,9 +330,9 @@ def preserve_older_revision(branch, remote_to_push):
     try:
         _run_shell_command(["git", "push", remote_to_push, tag])
     except RuntimeError:
-        LOG.error("Unable to push tag {} for previous revision".format(tag))
+        LOG.error("Unable to push tag %s for previous revision", tag)
     else:
-        LOG.info("Previous revision of patchset saved on tag {}".format(tag))
+        LOG.info("Previous revision of patchset saved on tag %s", tag)
 
     _run_shell_command(["git", "tag", "-d", tag])
 
@@ -320,23 +340,27 @@ def preserve_older_revision(branch, remote_to_push):
 def fork_and_push_pull_request(g, repo_to_fork, rebase, target_remote,
                                target_branch, branch, user, title, message,
                                comment_on_update, comment,
-                               force_editor, ignore_tag):
+                               force_editor, ignore_tag, dont_fork):
 
     g_user = g.get_user()
 
-    repo_forked = g_user.create_fork(repo_to_fork)
-    LOG.info("Forked repository: %s", repo_forked.html_url)
-
-    remote_to_push = git_remote_matching_url(repo_forked.clone_url)
-
-    if remote_to_push:
-        LOG.debug("Found forked repository already in remote as `%s'",
-                  remote_to_push)
+    if dont_fork:
+        remote_to_push = target_remote
     else:
-        remote_to_push = "github"
-        _run_shell_command(
-            ["git", "remote", "add", remote_to_push, repo_forked.clone_url])
-        LOG.info("Added forked repository as remote `%s'", remote_to_push)
+        repo_forked = g_user.create_fork(repo_to_fork)
+        LOG.info("Forked repository: %s", repo_forked.html_url)
+
+        remote_to_push = git_remote_matching_url(repo_forked.clone_url)
+
+        if remote_to_push:
+            LOG.debug("Found forked repository already in remote as `%s'",
+                      remote_to_push)
+        else:
+            remote_to_push = "github"
+            _run_shell_command(
+                ["git", "remote", "add",
+                 remote_to_push, repo_forked.clone_url])
+            LOG.info("Added forked repository as remote `%s'", remote_to_push)
 
     if rebase:
         _run_shell_command(["git", "remote", "update", target_remote])
@@ -397,19 +421,18 @@ def fork_and_push_pull_request(g, repo_to_fork, rebase, target_remote,
             LOG.debug("Commented: \"%s\"", comment)
     else:
         # Create a pull request
-        nb_of_commits, git_title, git_message = git_get_title_and_message(
-            "%s/%s" % (target_remote, target_branch), branch)
-
-        title = title or git_title
-        message = message or git_message
-
-        # Do not run an editor if there's only one commit or if both title and
-        # message were specified
-        if (force_editor or
-           ((not title or not message) and nb_of_commits > 1) or
-           (((title and not message) or
-             (message and not title)) and nb_of_commits == 1)):
-            title, message = edit_title_and_message(title, message)
+        if force_editor or not (title and message):
+            nb_of_commits, git_title, git_message = git_get_title_and_message(
+                "%s/%s" % (target_remote, target_branch), branch)
+            title = title or git_title
+            message = message or git_message
+            # Do not run an editor if there's only one commit or if both
+            # title and message were specified
+            if (force_editor or
+               ((not title or not message) and nb_of_commits > 1) or
+               (((title and not message) or
+                 (message and not title)) and nb_of_commits == 1)):
+                title, message = edit_title_and_message(title, message)
 
         if title is None:
             LOG.critical("Pull-request message is empty, aborting")
@@ -426,22 +449,22 @@ def fork_and_push_pull_request(g, repo_to_fork, rebase, target_remote,
             )
             return 50
         else:
-            LOG.info("Pull-request created: " + pull.html_url)
+            LOG.info("Pull-request created: %s", pull.html_url)
 
     if not ignore_tag:
         preserve_older_revision(branch, remote_to_push)
 
 
 def _format_github_exception(action, exc):
-        url = exc.data.get("documentation_url", "GitHub documentation")
-        errors_msg = "\n".join(map(operator.itemgetter("message"),
-                                   exc.data.get("errors", [])))
-        return (
-            "Unable to %s: %s (%s)\n"
-            "%s\n"
-            "Check %s for more information." %
-            (action, exc.data.get('message'), exc.status, errors_msg, url)
-        )
+    url = exc.data.get("documentation_url", "GitHub documentation")
+    errors_msg = "\n".join(map(operator.itemgetter("message"),
+                               exc.data.get("errors", [])))
+    return (
+        "Unable to %s: %s (%s)\n"
+        "%s\n"
+        "Check %s for more information." %
+        (action, exc.data.get('message'), exc.status, errors_msg, url)
+    )
 
 
 def main():
@@ -488,6 +511,12 @@ def main():
         default=False,
         help="Preserve older revision when pushing"
     )
+    parser.add_argument(
+        "--no-fork",
+        action="store_true",
+        default=False,
+        help="Don't fork to create the pull-request"
+    )
 
     args = parser.parse_args()
 
@@ -511,12 +540,11 @@ def main():
             rebase=not args.no_rebase,
             force_editor=args.force_editor,
             download=args.download,
-            ignore_tag=args.no_tag_previous_revision
+            ignore_tag=args.no_tag_previous_revision,
+            dont_fork=args.no_fork
         )
-    except Exception as e:
-        if args.debug:
-            raise
-        LOG.error(e)
+    except Exception:
+        LOG.error("Unable to send pull request", exc_info=True)
 
 
 if __name__ == '__main__':
